#This is a python script that creates an FEM model referred to as the Curved Model
#in the master thesis. Under the section "Define variables", the user can specify for example 
#the geometry or mesh density.

#Written by Erik Grimsmo

#------------------------------------------------------Initialize---------------------------------------------------------------


from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *

session.journalOptions.setValues(replayGeometry=COORDINATE,recoverGeometry=COORDINATE)

#Change work directory (use your personal directory, and uncomment the line below)
#os.chdir(r'D:\abaqusworkdirectory')

#----------------------------------------------Define variables--------------------------------------------------------------

a = 0.025 #Short axis of ellipse
b = 0.045 #Long axis of ellipse
c = 0.015 #Dimension used to define height of ellipse
th = 0.01 #Thickness (of heart wall)
d = 0.01 #Depth of extrusion

numberOfPartitions = 10 #Number of partitions of the myocardium 
#through it thickness (select even number)

rho = 1060.0 #Density

F = 1.0 #The amplitude of the applied traction load

t = 0.0005 #Period of time integration
c_t = 100 #Approx longitudinal wave speed
loadTime = 1E-4 #Duration of blast load
N = 20 #Number of elements over which the load will take place
elSize = loadTime*c_t/n #Element size

#Do not alter the script after this section, 
#unless you are familiar with python scripting in Abaqus

#----------------------------------------------Create model---------------------------------------------------------------

eps = 0.0001 #A variable that helps with modeling
r = sqrt(1-(c/b)**2)*a #A dimension that is used a lot

mdb.Model(name='CM')
mdb.models['CM'].setValues(noPartsInputFile=ON)

#----------------------------------------------Create sketch---------------------------------------------------------------

mdb.models['CM'].ConstrainedSketch(name='__profile__', sheetSize=1.0)
mdb.models['CM'].sketches['__profile__'].ConstructionLine(point1=(0.0, 
    -0.5), point2=(0.0, 0.5))

mdb.models['CM'].sketches['__profile__'].EllipseByCenterPerimeter(
    axisPoint1=(0.0, -b), axisPoint2=(a, 0.0), center=(0.0, 0.0))
mdb.models['CM'].sketches['__profile__'].EllipseByCenterPerimeter(
    axisPoint1=(0.0, -b+th), axisPoint2=(a-th, 0.0), center=(0.0, 0.0))
mdb.models['CM'].sketches['__profile__'].Line(point1=(0.0, 
    -b+th), point2=(0.0, -b))
mdb.models['CM'].sketches['__profile__'].Line(point1=(
    sqrt(1-(c/(b-th))**2)*(a-th), c), point2=(r, c))

mdb.models['CM'].sketches['__profile__'].geometry.findAt((-a, 0.0))
mdb.models['CM'].sketches['__profile__'].autoTrimCurve(curve1=
    mdb.models['CM'].sketches['__profile__'].geometry.findAt((-a, 
    0.0), ), point1=(-a, 0.0))
mdb.models['CM'].sketches['__profile__'].geometry.findAt((-a+th, 0.0))
mdb.models['CM'].sketches['__profile__'].autoTrimCurve(curve1=
    mdb.models['CM'].sketches['__profile__'].geometry.findAt((-a+th, 0.0), ),
    point1=(-a+th, 0.0))
mdb.models['CM'].sketches['__profile__'].geometry.findAt((0, b))
mdb.models['CM'].sketches['__profile__'].autoTrimCurve(curve1=
    mdb.models['CM'].sketches['__profile__'].geometry.findAt((0, b), ), 
    point1=(0, b))
mdb.models['CM'].sketches['__profile__'].geometry.findAt((0, b-th))
mdb.models['CM'].sketches['__profile__'].autoTrimCurve(curve1=
    mdb.models['CM'].sketches['__profile__'].geometry.findAt((0, b-th), ), 
    point1=(0, b-th))

mdb.models['CM'].Part(dimensionality=THREE_D, name='Part-1', type=
    DEFORMABLE_BODY)
mdb.models['CM'].parts['Part-1'].BaseSolidExtrude(depth=d, sketch=
    mdb.models['CM'].sketches['__profile__'])
del mdb.models['CM'].sketches['__profile__']

#----------------------------------------------Define material------------------------------------------------------------

#Isotropic material
mdb.models['CM'].Material(name='isotropic')
mdb.models['CM'].materials['isotropic'].Density(table=((1060.0, ), ))
mdb.models['CM'].materials['isotropic'].Elastic(table=((31588000.0, 0.49), ))

#Transverse isotropic material
mdb.models['CM'].Material(name='anisotropic')
mdb.models['CM'].materials['anisotropic'].Elastic(table=((2460000000.0, 
    2443100000.0, 2460000000.0, 2440000000.0, 2440000000.0, 2530000000.0, 0.0, 
    0.0, 0.0, 8970000.0, 0.0, 0.0, 0.0, 0.0, 8970000.0, 0.0, 0.0, 0.0, 0.0, 
    0.0, 8450000.0), ), type=ANISOTROPIC)
mdb.models['CM'].materials['anisotropic'].Density(table=((rho, ), ))

#----------------------------------------------Create section---------------------------------------------------------------

#Choose here what material you want to work with
mdb.models['CM'].HomogeneousSolidSection(material='anisotropic', name=
    'Section-1', thickness=None)
mdb.models['CM'].parts['Part-1'].SectionAssignment(offset=0.0, offsetField=''
    , offsetType=MIDDLE_SURFACE, region=Region(
    cells=mdb.models['CM'].parts['Part-1'].cells.findAt(((a, 0, 
    0), ), )), sectionName='Section-1', thicknessAssignment=
    FROM_SECTION)

#------------------------------------Partition thickness of the model--------------------------------------------------
    
mdb.models['CM'].ConstrainedSketch(gridSpacing=0.02, name='__profile__', 
    sheetSize=1.07, transform=
    mdb.models['CM'].parts['Part-1'].MakeSketchTransform(
    sketchPlane=mdb.models['CM'].parts['Part-1'].faces.findAt((a-th/2, 
    0.0, 0.0), ), sketchPlaneSide=SIDE1, 
    sketchUpEdge=mdb.models['CM'].parts['Part-1'].edges.findAt((0.0, -b+th/2, 
    0.0), ), sketchOrientation=RIGHT, origin=(0.0, 0.0, 0.0)))
mdb.models['CM'].parts['Part-1'].projectReferencesOntoSketch(filter=
    COPLANAR_EDGES, sketch=mdb.models['CM'].sketches['__profile__'])

l1 = th/numberOfPartitions #Length of one partition

for i in range(numberOfPartitions-1):
    mdb.models['CM'].sketches['__profile__'].EllipseByCenterPerimeter(
        axisPoint1=(0.0, b-(i+1)*l1), axisPoint2=(a-(i+1)*l1, 0.0), center=(0.0, 0.0))

mdb.models['CM'].parts['Part-1'].PartitionFaceBySketch(faces=
    mdb.models['CM'].parts['Part-1'].faces.findAt(((a-th/2, 0.0, 
    0.0), )), sketch=mdb.models['CM'].sketches['__profile__'], 
    sketchUpEdge=mdb.models['CM'].parts['Part-1'].edges.findAt((0.0, -b+th/2, 
    0.0), ))

for i in range(numberOfPartitions-1):
    mdb.models['CM'].parts['Part-1'].PartitionCellBySweepEdge(cells=
        mdb.models['CM'].parts['Part-1'].cells.findAt(((a, 0.0, 
        d/2), )), edges=(mdb.models['CM'].parts['Part-1'].edges.findAt((
        a-th+(i+1)*l1, 0.0, 0), ), ), sweepPath=
        mdb.models['CM'].parts['Part-1'].edges.findAt((r, c, 
        d/2), ))

#Finally, material orientation can be given to each cell
for i in range(numberOfPartitions):
    mdb.models['CM'].parts['Part-1'].MaterialOrientation(
        additionalRotationField='', additionalRotationType=ROTATION_ANGLE, angle=
        60-i*120/(numberOfPartitions-1), axis=AXIS_1, flipNormalDirection=False, flipPrimaryDirection=False, 
        normalAxisDefinition=SURFACE, normalAxisDirection=AXIS_1, normalAxisRegion=
        Region(side1Faces=mdb.models['CM'].parts['Part-1'].faces.findAt(((
        a, 0.0, d/2), ), )), orientationType=DISCRETE, 
        primaryAxisDefinition=EDGE, primaryAxisDirection=AXIS_3, primaryAxisRegion=
        Region(edges=mdb.models['CM'].parts['Part-1'].edges.findAt(((0.0, 
        -b, d/2), ), )), region=Region(
        cells=mdb.models['CM'].parts['Part-1'].cells.findAt(((a-i*l1-eps, 0.0, 
        d/2), ), )), stackDirection=STACK_3)


#----------------------------------------------Other partitions--------------------------------------------------------------
#The following partitions are created such that we can define sets where the stress
#vs. time will be extracted

myDatum1 = mdb.models['CM'].parts['Part-1'].DatumPlaneByPrincipalPlane(offset=0.0, 
    principalPlane=XZPLANE)
myDatum2 = mdb.models['CM'].parts['Part-1'].DatumPlaneByPrincipalPlane(offset=-c, 
    principalPlane=XZPLANE)
myDatum3 = mdb.models['CM'].parts['Part-1'].DatumPlaneByPrincipalPlane(offset=d/2, 
    principalPlane=XYPLANE)

mdb.models['CM'].parts['Part-1'].PartitionCellByDatumPlane(cells=
    mdb.models['CM'].parts['Part-1'].cells.getByBoundingBox(
    -1,-b-1,-1,a+1,c+1,d+1), datumPlane=
    mdb.models['CM'].parts['Part-1'].datums[myDatum1.id])
mdb.models['CM'].parts['Part-1'].PartitionCellByDatumPlane(cells=
    mdb.models['CM'].parts['Part-1'].cells.getByBoundingBox(
    -1,-b-1,-1,a+1,c+1,d+1), datumPlane=
    mdb.models['CM'].parts['Part-1'].datums[myDatum2.id])
mdb.models['CM'].parts['Part-1'].PartitionCellByDatumPlane(cells=
    mdb.models['CM'].parts['Part-1'].cells.getByBoundingBox(
    -1,-b-1,-1,a+1,c+1,d+1), datumPlane=
    mdb.models['CM'].parts['Part-1'].datums[myDatum3.id])

#Define the sets
mdb.models['CM'].parts['Part-1'].Set(name='node1b', vertices=
    mdb.models['CM'].parts['Part-1'].vertices.findAt(((a-th/2, 0.0, d/2), 
    )))
mdb.models['CM'].parts['Part-1'].Set(name='node1c', vertices=
    mdb.models['CM'].parts['Part-1'].vertices.findAt(((a-l1, 0.0, d/2), 
    )))
mdb.models['CM'].parts['Part-1'].Set(name='node1a', vertices=
    mdb.models['CM'].parts['Part-1'].vertices.findAt(((
    a-(numberOfPartitions-1)*l1, 0.0, d/2),  )))
mdb.models['CM'].parts['Part-1'].Set(name='node2', vertices=
    mdb.models['CM'].parts['Part-1'].vertices.findAt(((
    sqrt(1-(c/(b-numberOfPartitions/2*l1))**2)*(a-numberOfPartitions/2*l1), -c, 
    d/2), )))
    
#----------------------------------------------Create assembly--------------------------------------------------------------
    
mdb.models['CM'].rootAssembly.DatumCsysByDefault(CARTESIAN)
mdb.models['CM'].rootAssembly.Instance(dependent=OFF, name='CM-1', 
        part=mdb.models['CM'].parts['Part-1'])

    
#------------------------------------------------Create step-------------------------------------------------------------------
    
mdb.models['CM'].ExplicitDynamicsStep(name='dynamic', previous=
       'Initial', timePeriod=t,  quadBulkViscosity=1.2)
    
#-----------------------------------------Define load and BCs--------------------------------------------------------------
 
#Fixed end    
mdb.models['CM'].EncastreBC(createStepName='Initial', localCsys=None, name=
    'encastre', region=Region(
    faces=mdb.models['CM'].rootAssembly.instances['CM-1'].faces.getByBoundingBox(
    -eps,-b-eps,-eps,eps,-b+th+eps,d+eps)))

#Fix sides
mdb.models['CM'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='fix U1 and U2 1', 
    region=Region(
    faces=mdb.models['CM'].rootAssembly.instances['CM-1'].faces.getByBoundingBox(
    -1,-b-1,-eps,a+1,c+1,eps)), u1=SET, u2=SET, u3=UNSET, ur1=UNSET, 
    ur2=UNSET, ur3=UNSET)
mdb.models['CM'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='fix U1 and U2 2', 
    region=Region(
    faces=mdb.models['CM'].rootAssembly.instances['CM-1'].faces.getByBoundingBox(
    -1,-b-1,d-eps,a+1,c+1,d+eps)), u1=SET, u2=SET, u3=UNSET, ur1=UNSET, 
    ur2=UNSET, ur3=UNSET)

 #Determine the load shape in time
mdb.models['CM'].SmoothStepAmplitude(data=((0.0, 0.0), (1E-4-1E-5, 0.0), (
      1E-4, 1.0), (1E-4+loadTime, 1.0), (1E-4+loadTime+1E-5, 0.0)), name='impulse_smooth', timeSpan=STEP)

#Apply surface traction
mdb.models['CM'].SurfaceTraction(amplitude='impulse_smooth', 
    createStepName='dynamic', directionVector=(
    mdb.models['CM'].rootAssembly.instances['CM-1'].vertices.findAt((
    a, 0.0, 0.0), ), 
    mdb.models['CM'].rootAssembly.instances['CM-1'].vertices.findAt((
    a, 0.0, d), )), distributionType=UNIFORM, field='', localCsys=None
    , magnitude=F, name='shear', region=Region(
    side1Faces=mdb.models['CM'].rootAssembly.instances['CM-1'].faces.getByBoundingBox(
    0,c-eps,-1,a+1,c+eps,d+1 )))

#----------------------------------------------Generate mesh-------------------------------------------------------------

mdb.models['CM'].rootAssembly.seedPartInstance(deviationFactor=0.1, 
    minSizeFactor=0.1, regions=(
    mdb.models['CM'].rootAssembly.instances['CM-1'], ), size=elSize)
mdb.models['CM'].rootAssembly.setElementType(elemTypes=(ElemType(
    elemCode=C3D8R, elemLibrary=EXPLICIT, secondOrderAccuracy=OFF, 
    kinematicSplit=AVERAGE_STRAIN, hourglassControl=DEFAULT, 
    distortionControl=DEFAULT), ElemType(elemCode=C3D6, elemLibrary=EXPLICIT), 
    ElemType(elemCode=C3D4, elemLibrary=EXPLICIT)), regions=(
    mdb.models['CM'].rootAssembly.instances['CM-1'].cells.getByBoundingBox(
    -1,-b-1,-1,a+1,c+1,d+1), ))
mdb.models['CM'].rootAssembly.generateMesh(regions=(
    mdb.models['CM'].rootAssembly.instances['CM-1'],))

#--------------------------------------------------Request outputs---------------------------------------------------------

mdb.models['CM'].fieldOutputRequests['F-Output-1'].setValues(directions=OFF, 
    numIntervals=400, variables=('S', 'SVAVG',  'U','ENER' ,))

#----------------------------------------------Create job ---------------------------------------------------------------------

mdb.models['CM'].rootAssembly.regenerate()
mdb.Job(activateLoadBalancing=False, atTime=None, contactPrint=OFF, 
    description='', echoPrint=OFF, explicitPrecision=SINGLE, historyPrint=OFF, 
    model='CM', modelPrint=OFF, multiprocessingMode=DEFAULT, name=
    'Curved_Model', nodalOutputPrecision=SINGLE, numCpus=1, numDomains=1, 
    parallelizationMethodExplicit=DOMAIN, queue=None, scratch='', type=ANALYSIS
    , userSubroutine='', waitHours=0, waitMinutes=0)


