#This is a python script that creates an FEM model referred to as the Finite-sized Wave Model
#in the master thesis. Under the section "Define variables", the user can specify for example 
#the geometry or mesh density.

#Written by Erik Grimsmo

#------------------------------------------------------Initialize---------------------------------------------------------------

from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *

session.journalOptions.setValues(replayGeometry=COORDINATE,recoverGeometry=COORDINATE)

#Change work directory (use your personal directory, and uncomment the line below)
#os.chdir(r'D:\abaqusworkdirectory')

#----------------------------------------------Define variables--------------------------------------------------------------

a = 0.1#Height of plate
b = 0.4 #Length of plate
c = 0.04 #Length of half  the domain where the load is applied

theta = 45  #Orientation of plate. 0 degrees = n aligned with fibre direction

rho = 1060.0 #Density

F = 1.0 #The amplitude of the applied traction load

t = 0.001 #Period of time integration
c_t = 100 #Approx.  shear wave speed, used to find appropriate element size
loadTime = 1E-4 #Duration of blast load
N = 10 #Number of elements over which the load will take place
elSize = loadTime*c_t/n #Element size

#Do not alter the script after this section, 
#unless you are familiar with python scripting in Abaqus

#----------------------------------------------Create model---------------------------------------------------------------

eps = 0.0001 #A variable that helps with modeling
theta = 90-theta #The model was initially made with  theta
#equal to zero along x1-direction. This is fixed with this line.

mdb.Model(name='FSWM')
mdb.models['FSWM'].setValues(noPartsInputFile=ON)

#----------------------------------------------Create sketch---------------------------------------------------------------

mdb.models['FSWM'].ConstrainedSketch(name='plateSketch', sheetSize=200.0)
mdb.models['FSWM'].sketches['plateSketch'].rectangle(point1=(0.0, 0.0), 
    point2=(a, b))
mdb.models['FSWM'].Part(dimensionality=TWO_D_PLANAR, name='plate', 
    type=DEFORMABLE_BODY)
mdb.models['FSWM'].parts['plate'].BaseShell(sketch=
    mdb.models['FSWM'].sketches['plateSketch'])

#----------------------------------------------Define material------------------------------------------------------------

#Isotropic material
mdb.models['FSWM'].Material(name='isotropic')
mdb.models['FSWM'].materials['isotropic'].Density(table=((1060.0, ), ))
mdb.models['FSWM'].materials['isotropic'].Elastic(table=((39765628.0, 0.49), ))

#Transverse isotropic material
mdb.models['FSWM'].Material(name='anisotropic')
mdb.models['FSWM'].materials['anisotropic'].Elastic(table=((2460000000.0, 
    2440000000.0, 2530000000.0, 2443100000.0, 2440000000.0, 2460000000.0, 0.0, 
    0.0, 0.0, 8970000.0, 0.0, 0.0, 0.0, 0.0, 8450000.0, 0.0, 0.0, 0.0, 0.0, 
    0.0, 8970000), ), type=ANISOTROPIC)
mdb.models['FSWM'].materials['anisotropic'].Density(table=((rho, ), ))
mdb.models['FSWM'].parts['plate'].MaterialOrientation(
    additionalRotationField='', additionalRotationType=ROTATION_ANGLE, angle=
    0, axis=AXIS_3, fieldName='', localCsys=None, orientationType=SYSTEM, 
    region=Region(
    faces=mdb.models['FSWM'].parts['plate'].faces.getByBoundingBox(
    -1,-1,-1,a+1,b+1,1) ,), stackDirection=STACK_3)

#----------------------------------------------Create section---------------------------------------------------------------

#Choose here what material you want to work with
mdb.models['FSWM'].HomogeneousSolidSection(material='anisotropic', name=
    'Section-1', thickness=None)
mdb.models['FSWM'].parts['plate'].SectionAssignment(offset=0.0, 
    offsetField='', offsetType=MIDDLE_SURFACE, region=Region(
    faces=mdb.models['FSWM'].parts['plate'].faces.getSequenceFromMask(
    mask=('[#1 ]', ), )), sectionName='Section-1', thicknessAssignment=
    FROM_SECTION)

#----------------------------------------------Partition face--------------------------------------------------------------

#We apply the partitions in the section, because want the partition where the load is to be 
#applied.

mdb.models['FSWM'].ConstrainedSketch(gridSpacing=0.001, name='__profile__', 
    sheetSize=0.044, transform=
    mdb.models['FSWM'].parts['plate'].MakeSketchTransform(
    sketchPlane=mdb.models['FSWM'].parts['plate'].faces.findAt((
    a/2, b/2, 0.0), (0.0, 0.0, 1.0)), sketchPlaneSide=SIDE1, 
    sketchOrientation=RIGHT, origin=(0.0, 0.0, 0.0)))
mdb.models['FSWM'].parts['plate'].projectReferencesOntoSketch(filter=
    COPLANAR_EDGES, sketch=mdb.models['FSWM'].sketches['__profile__'])
mdb.models['FSWM'].sketches['__profile__'].Line(point1=(0.0, b/2), point2=(
    a, b/2))
mdb.models['FSWM'].sketches['__profile__'].Line(point1=(0.0, b/2+c), point2=(
    a, b/2+c))
mdb.models['FSWM'].sketches['__profile__'].Line(point1=(a, b/2-c), point2=(
    0.0, b/2-c))
mdb.models['FSWM'].parts['plate'].PartitionFaceBySketch(faces=
    mdb.models['FSWM'].parts['plate'].faces.findAt(((a/2, 
    b/2, 0.0), )), sketch=mdb.models['FSWM'].sketches['__profile__'])
del mdb.models['FSWM'].sketches['__profile__']


#----------------------------------------------Create assembly--------------------------------------------------------------

mdb.models['FSWM'].rootAssembly.DatumCsysByDefault(CARTESIAN)
mdb.models['FSWM'].rootAssembly.Instance(dependent=ON, name='plate-1', 
    part=mdb.models['FSWM'].parts['plate'])

#------------------------------------------------Create step-------------------------------------------------------------------

mdb.models['FSWM'].ExplicitDynamicsStep(name='dynamic',previous=
    'Initial', timePeriod=t,  quadBulkViscosity=0.0)

#-----------------------------------------Define load and BCs--------------------------------------------------------------

#Determine the load shape in time
mdb.models['FSWM'].SmoothStepAmplitude(data=((0.0, 0.0), (loadTime-1E-5, 0.0), (
    loadTime, 1.0), (2*loadTime, 1.0), (2*loadTime+1E-5, 0.0)), name='impulse_smooth', timeSpan=STEP)
    
#Fixed end    
mdb.models['FSWM'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='fixedEnd', 
    region=Region(
    edges=mdb.models['FSWM'].rootAssembly.instances['plate-1'].edges.findAt(
    ((a, b-eps, 0.0), ), ((a,eps,0.0),),((a, b/2-eps, 0.0), ), ((a,b/2+eps,0.0),),
    )), u1=SET, u2=SET, ur3=SET)

#Apply surface traction
mdb.models['FSWM'].SurfaceTraction(amplitude='impulse_smooth', createStepName=
    'dynamic', directionVector=(
    mdb.models['FSWM'].rootAssembly.instances['plate-1'].InterestingPoint(
    mdb.models['FSWM'].rootAssembly.instances['plate-1'].edges.findAt(
    (a, 0.0, 0.0), ), MIDDLE), 
    mdb.models['FSWM'].rootAssembly.instances['plate-1'].vertices.findAt(
    (a, b/2, 0.0), )), distributionType=UNIFORM, field='', localCsys=None, 
    magnitude=F, name='traction', region=Region(
    side1Edges=mdb.models['FSWM'].rootAssembly.instances['plate-1'].edges.findAt(
    ((0, b/2+eps, 0.0), ), ((0, b/2-eps, 0.0), ), )), resultant=ON)

#----------------------------------------------Generate mesh-------------------------------------------------------------

mdb.models['FSWM'].parts['plate'].seedPart(deviationFactor=0.1, 
    minSizeFactor=0.1, size=elSize)
mdb.models['FSWM'].parts['plate'].setMeshControls(elemShape=QUAD, 
    regions=mdb.models['FSWM'].parts['plate'].faces.getByBoundingBox( 
    -1,-1,-1,a+1,b+1,2 ), technique=STRUCTURED)
mdb.models['FSWM'].parts['plate'].setElementType(elemTypes=(ElemType(
    elemCode=CPE4R, elemLibrary=STANDARD, secondOrderAccuracy=OFF, 
    hourglassControl=DEFAULT, distortionControl=DEFAULT), ElemType(
    elemCode=CPE4R, elemLibrary=STANDARD)), regions=(
    mdb.models['FSWM'].parts['plate'].faces.getByBoundingBox( 
    -1,-1,-1,a+1,b+1,2 ),))
mdb.models['FSWM'].parts['plate'].generateMesh()

#-----------------------------------------------Rotate plate---------------------------------------------------------------

mdb.models['FSWM'].rootAssembly.rotate(angle=theta, axisDirection=(0.0, 0.0, 
    1.0), axisPoint=(0.0, 0.0, 0.0), instanceList=('plate-1', ))

#--------------------------------------------------Request outputs---------------------------------------------------------

#Field output
mdb.models['FSWM'].fieldOutputRequests['F-Output-1'].setValues(numIntervals=
    300,  directions=OFF, variables=('S', 'ENER','SVAVG', 'U', 'V', 'A'))

#----------------------------------------------Create job ---------------------------------------------------------------------

mdb.models['FSWM'].rootAssembly.regenerate()
mdb.Job(activateLoadBalancing=False, atTime=None, contactPrint=OFF, 
    description='', echoPrint=OFF, explicitPrecision=SINGLE, historyPrint=OFF, 
    model='FSWM', modelPrint=OFF, multiprocessingMode=DEFAULT, name=
    'Finite-sized_Wave_Model', nodalOutputPrecision=SINGLE, numCpus=1, numDomains=
    1, parallelizationMethodExplicit=DOMAIN, queue=None, scratch='', type=
    ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)

