#This is a python script that creates an FEM model referred to as the Plane Wave Model
#in the master thesis. Under the section "Define variables", the user can specify for example 
#the geometry or mesh density.

#Written by Erik Grimsmo

#------------------------------------------------------Initialize---------------------------------------------------------------

from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *

session.journalOptions.setValues(replayGeometry=COORDINATE,recoverGeometry=COORDINATE)

#Change work directory (use your personal directory, and uncomment the line below)
#os.chdir(r'D:\abaqusworkdirectory')

#----------------------------------------------Define variables--------------------------------------------------------------

a = 0.2 #Height of plate
b = 0.2 #Length of plate

theta = 45 #Orientation of plate. 0 degrees = n aligned with fibre direction

rho = 1060.0 #Density

F = 1.0 #The amplitude of the applied traction load

t = 0.0015 #Period of time integration
c_t = 100.0 #Approx. transverse wave speed, used to find appropriate element size
loadTime = 1E-4 #Duration of blast load
N = 10 #Number of elements over which the load will take place
elSize = loadTime*c_t/n #Element size

#Do not alter the script after this section, 
#unless you are familiar with python scripting in Abaqus

#----------------------------------------------Create model---------------------------------------------------------------

theta = 90-theta #The model was initially made with  theta
#equal to zero along x1-direction. This is fixed with this line.

mdb.Model(name='PWM')
mdb.models['PWM'].setValues(noPartsInputFile=ON)

#----------------------------------------------Create sketch---------------------------------------------------------------

mdb.models['PWM'].ConstrainedSketch(name='plateSketch', sheetSize=200.0)
mdb.models['PWM'].sketches['plateSketch'].rectangle(point1=(0.0, 0.0), 
    point2=(a, b))
mdb.models['PWM'].Part(dimensionality=TWO_D_PLANAR, name='plate', 
    type=DEFORMABLE_BODY)
mdb.models['PWM'].parts['plate'].BaseShell(sketch=
    mdb.models['PWM'].sketches['plateSketch'])

#----------------------------------------------Define material------------------------------------------------------------

#Isotropic material
mdb.models['PWM'].Material(name='isotropic')
mdb.models['PWM'].materials['isotropic'].Density(table=((1060.0, ), ))
mdb.models['PWM'].materials['isotropic'].Elastic(table=((39765628.0, 0.49), ))

#Transverse isotropic material
mdb.models['PWM'].Material(name='anisotropic')
mdb.models['PWM'].materials['anisotropic'].Elastic(table=((2460000000.0, 
    2440000000.0, 2530000000.0, 2443100000.0, 2440000000.0, 2460000000.0, 0.0, 
    0.0, 0.0, 8970000.0, 0.0, 0.0, 0.0, 0.0, 8450000.0, 0.0, 0.0, 0.0, 0.0, 
    0.0, 8970000), ), type=ANISOTROPIC)
mdb.models['PWM'].materials['anisotropic'].Density(table=((rho, ), ))
mdb.models['PWM'].parts['plate'].MaterialOrientation(
    additionalRotationField='', additionalRotationType=ROTATION_ANGLE, angle=
    0, axis=AXIS_3, fieldName='', localCsys=None, orientationType=SYSTEM, 
    region=Region(
    faces=mdb.models['PWM'].parts['plate'].faces.getByBoundingBox(
    -1,-1,-1,a+1,b+1,1) ,), stackDirection=STACK_3)

#----------------------------------------------Create section---------------------------------------------------------------

#Choose here what material you want to work with
mdb.models['PWM'].HomogeneousSolidSection(material='anisotropic', name=
    'Section-1', thickness=None)
mdb.models['PWM'].parts['plate'].SectionAssignment(offset=0.0, 
    offsetField='', offsetType=MIDDLE_SURFACE, region=Region(
    faces=mdb.models['PWM'].parts['plate'].faces.getSequenceFromMask(
    mask=('[#1 ]', ), )), sectionName='Section-1', thicknessAssignment=
    FROM_SECTION)

#----------------------------------------------Partition face--------------------------------------------------------------

#Here we partition the face that allow defining a set at the  center of the model

mdb.models['PWM'].ConstrainedSketch(gridSpacing=0.001, name='__profile__', 
    sheetSize=0.044, transform=
    mdb.models['PWM'].parts['plate'].MakeSketchTransform(
    sketchPlane=mdb.models['PWM'].parts['plate'].faces.findAt((
    a/2, b/2, 0.0), (0.0, 0.0, 1.0)), sketchPlaneSide=SIDE1, 
    sketchOrientation=RIGHT, origin=(0.0, 0.0, 0.0)))
mdb.models['PWM'].parts['plate'].projectReferencesOntoSketch(filter=
    COPLANAR_EDGES, sketch=mdb.models['PWM'].sketches['__profile__'])
mdb.models['PWM'].sketches['__profile__'].Line(point1=(0.0, b/2), point2=(
    a, b/2))
mdb.models['PWM'].sketches['__profile__'].Line(point1=(a/2, 0.0), point2=(
    a/2, b))
mdb.models['PWM'].parts['plate'].PartitionFaceBySketch(faces=
    mdb.models['PWM'].parts['plate'].faces.findAt(((a/2, 
    b/2, 0.0), )), sketch=mdb.models['PWM'].sketches['__profile__'])
del mdb.models['PWM'].sketches['__profile__']


#----------------------------------------------Create assembly--------------------------------------------------------------

mdb.models['PWM'].rootAssembly.DatumCsysByDefault(CARTESIAN)
mdb.models['PWM'].rootAssembly.Instance(dependent=ON, name='plate-1', 
    part=mdb.models['PWM'].parts['plate'])

#------------------------------------------------Create step-------------------------------------------------------------------

mdb.models['PWM'].ExplicitDynamicsStep(name='dynamic',previous=
    'Initial', timePeriod=t,  quadBulkViscosity=0.0, linearBulkViscosity=0.06)

#------------------------------------------Apply BCs and load---------------------------------------------------------------

#Fixed end        
mdb.models['PWM'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='fixedEnd', 
    region=Region(
    edges=mdb.models['PWM'].rootAssembly.instances['plate-1'].edges.findAt(
    ((a, b/4, 0.0), ), ((a,3*b/4,0.0),), )), u1=SET, u2=SET, ur3=SET)
    
#Determine the load shape in time
mdb.models['PWM'].SmoothStepAmplitude(data=((0.0, 0.0), (loadTime-1E-5, 0.0), (
    loadTime, 1.0), (2*loadTime, 1.0), (2*loadTime+1E-5, 0.0)), name='impulse_smooth', timeSpan=STEP)

#Apply surface traction
mdb.models['PWM'].SurfaceTraction(amplitude='impulse_smooth', createStepName=
    'dynamic', directionVector=(
    mdb.models['PWM'].rootAssembly.instances['plate-1'].InterestingPoint(
    mdb.models['PWM'].rootAssembly.instances['plate-1'].edges.findAt(
    (a, 0.0, 0.0), ), MIDDLE), 
    mdb.models['PWM'].rootAssembly.instances['plate-1'].vertices.findAt(
    (a, b/2, 0.0), )), distributionType=UNIFORM, field='', localCsys=None, 
    magnitude=F, name='traction', region=Region(
    side1Edges=mdb.models['PWM'].rootAssembly.instances['plate-1'].edges.findAt(
    ((0, b/4, 0.0), ), ((0, 3*b/4, 0.0), ), )), resultant=ON)

#----------------------------------------------Generate mesh-------------------------------------------------------------

mdb.models['PWM'].parts['plate'].seedPart(deviationFactor=0.1, 
    minSizeFactor=0.1, size=elSize)
mdb.models['PWM'].parts['plate'].setMeshControls(elemShape=QUAD, 
    regions=mdb.models['PWM'].parts['plate'].faces.findAt(((a/4, 
    b/4, 0.0),), ((3*a/4,b/4,0),), ((a/4,3*b/4,0),), ((3*a/4,3*b/4,0),) ), technique=STRUCTURED)
mdb.models['PWM'].parts['plate'].setElementType(elemTypes=(ElemType(
    elemCode=CPE4R, elemLibrary=STANDARD, secondOrderAccuracy=OFF, 
    hourglassControl=DEFAULT, distortionControl=DEFAULT), ElemType(
    elemCode=CPE4R, elemLibrary=STANDARD)), regions=(
    mdb.models['PWM'].parts['plate'].faces.findAt(((a/4, 
    b/4, 0.0),), ((3*a/4,b/4,0),), ((a/4,3*b/4,0),), ((3*a/4,3*b/4,0),),),))
mdb.models['PWM'].parts['plate'].generateMesh()

#----------------------------Rotate plate and apply boundary condititions---------------------------------------

mdb.models['PWM'].rootAssembly.rotate(angle=theta, axisDirection=(0.0, 0.0, 
    1.0), axisPoint=(0.0, 0.0, 0.0), instanceList=('plate-1', ))

#Apply boundary conditions that can only be applied after rotation
theta = theta*pi/180

myDatum = mdb.models['PWM'].rootAssembly.DatumCsysByThreePoints(coordSysType=CARTESIAN
    , name='Datum csys-2', origin=
    mdb.models['PWM'].rootAssembly.instances['plate-1'].vertices.findAt(
    (0.0, 0.0, 0.0), ), point1=
    mdb.models['PWM'].rootAssembly.instances['plate-1'].vertices.findAt(
    (a*cos(theta), a*sin(theta), 0.0), ), point2=
    mdb.models['PWM'].rootAssembly.instances['plate-1'].vertices.findAt(
    (-b*sin(theta), b*cos(theta), 0.0), ))

mdb.models['PWM'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', 
    localCsys=mdb.models['PWM'].rootAssembly.datums[myDatum.id], name='fixedEnd', 
    region=Region(
    edges=mdb.models['PWM'].rootAssembly.instances['plate-1'].edges.findAt(
    ((a*cos(theta)-0.25*b*sin(theta), a*sin(theta)+0.25*b*cos(theta), 0.0), ), 
    ((a*cos(theta)-0.75*b*sin(theta), a*sin(theta)+0.75*b*cos(theta), 0.0), ), )),
    u1=SET, u2=SET, ur3=SET)

mdb.models['PWM'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=
    mdb.models['PWM'].rootAssembly.datums[myDatum.id], name='rolling support', region=Region(
    edges=mdb.models['PWM'].rootAssembly.instances['plate-1'].edges.findAt(
    ((0.25*a*cos(theta), 0.25*a*sin(theta), 0.0), ),
    ((0.75*a*cos(theta), 0.75*a*sin(theta), 0.0), ),
    ((0.25*a*cos(theta)-b*sin(theta), 0.25*a*sin(theta)+b*cos(theta), 0.0), ),
    ((0.75*a*cos(theta)-b*sin(theta),  0.75*a*sin(theta)+b*cos(theta), 0.0), ),
    )), u1=SET, u2=UNSET, ur3=UNSET)

#--------------------------------------------------Request outputs---------------------------------------------------------

#Define set at center and at load of plate
mdb.models['PWM'].parts['plate'].Set(name='midNode', vertices=
    mdb.models['PWM'].parts['plate'].vertices.findAt(((a/2, b/2, 
    0.0), )))

#History output at midnode
mdb.models['PWM'].historyOutputRequests['H-Output-1'].setValues(frequency=1, 
    rebar=EXCLUDE, region=
    mdb.models['PWM'].rootAssembly.instances['plate-1'].sets['midNode']
    , sectionPoints=DEFAULT, variables=('UT', 'S12', ))

#Field output
mdb.models['PWM'].fieldOutputRequests['F-Output-1'].setValues(numIntervals=
    300,  directions=OFF, variables=('S', ))

#----------------------------------------------Create job ---------------------------------------------------------------------

mdb.models['PWM'].rootAssembly.regenerate()
mdb.Job(activateLoadBalancing=False, atTime=None, contactPrint=OFF, 
    description='', echoPrint=OFF, explicitPrecision=SINGLE, historyPrint=OFF, 
    model='PWM', modelPrint=OFF, multiprocessingMode=DEFAULT, name=
    'Plane_Wave_Model', nodalOutputPrecision=SINGLE, numCpus=1, numDomains=
    1, parallelizationMethodExplicit=DOMAIN, queue=None, scratch='', type=
    ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)