#This is a python script that creates an FEM model referred to as the Truncated Ellipsoid Model
#in the master thesis. Under the section "Define variables", the user can specify for example 
#the geometry or mesh density.

#Written by Erik Grimsmo

#------------------------------------------------------Initialize---------------------------------------------------------------

from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *

session.journalOptions.setValues(replayGeometry=COORDINATE,recoverGeometry=COORDINATE)

#Change work directory (use your personal directory, and uncomment the line below)
#os.chdir(r'D:\abaqusworkdirectory')

#----------------------------------------------Define variables--------------------------------------------------------------

a = 0.025 #Short axis of ellipsoid
b = 0.045 #Long axis of ellipsoid
c = 0.015 #Dimension used to define height of ellipsoid
th = 0.01 #Thickness of heart wall

numberOfPartitions = 10 #Number of partitions of the myocardium 
#through it thickness (select even number)

rho = 1060.0 #Density

F = 1.0 #The amplitude of the applied traction load

t = 0.0005 #Period of time integration
c_t= 100 #Approx transverse wave speed
loadTime = 1E-4 #Duration of blast load
N = 10 #Number of elements over which the load will take place
elSize = loadTime*c_t/n #Element size

#Do not alter the script after this section, 
#unless you are familiar with python scripting in Abaqus

#----------------------------------------------Create model---------------------------------------------------------------

eps = 0.00001 #A variable that helps with modeling

mdb.Model(name='TEM')
mdb.models['TEM'].setValues(noPartsInputFile=ON)

#----------------------------------------------Create sketch---------------------------------------------------------------

mdb.models['TEM'].ConstrainedSketch(name='__profile__', sheetSize=1.0)
mdb.models['TEM'].sketches['__profile__'].ConstructionLine(point1=(0.0, 
    -0.5), point2=(0.0, 0.5))

mdb.models['TEM'].sketches['__profile__'].EllipseByCenterPerimeter(
    axisPoint1=(0.0, -b), axisPoint2=(a, 0.0), center=(0.0, 0.0))
mdb.models['TEM'].sketches['__profile__'].EllipseByCenterPerimeter(
    axisPoint1=(0.0, -b+th), axisPoint2=(a-th, 0.0), center=(0.0, 0.0))
mdb.models['TEM'].sketches['__profile__'].Line(point1=(0, 
    -b), point2=(0, -b+th))
mdb.models['TEM'].sketches['__profile__'].Line(point1=(
    sqrt(1-(c/(b-th))**2)*(a-th), c), point2=(sqrt(1-(c/b)**2)*a, c))

mdb.models['TEM'].sketches['__profile__'].geometry.findAt((-a, 0.0))
mdb.models['TEM'].sketches['__profile__'].autoTrimCurve(curve1=
    mdb.models['TEM'].sketches['__profile__'].geometry.findAt((-a, 
    0.0), ), point1=(-a, 0.0))
mdb.models['TEM'].sketches['__profile__'].geometry.findAt((-a+th, 0.0))
mdb.models['TEM'].sketches['__profile__'].autoTrimCurve(curve1=
    mdb.models['TEM'].sketches['__profile__'].geometry.findAt((-a+th, 0.0), ),
    point1=(-a+th, 0.0))
mdb.models['TEM'].sketches['__profile__'].geometry.findAt((eps, b))
mdb.models['TEM'].sketches['__profile__'].autoTrimCurve(curve1=
    mdb.models['TEM'].sketches['__profile__'].geometry.findAt((eps, b), ), 
    point1=(eps, b))
mdb.models['TEM'].sketches['__profile__'].geometry.findAt((eps, b-th))
mdb.models['TEM'].sketches['__profile__'].autoTrimCurve(curve1=
    mdb.models['TEM'].sketches['__profile__'].geometry.findAt((eps, b-th), ), 
    point1=(eps, b-th))

mdb.models['TEM'].Part(dimensionality=THREE_D, name='ellipsoid', type=
    DEFORMABLE_BODY)
mdb.models['TEM'].parts['ellipsoid'].BaseSolidRevolve(angle=360.0, 
    flipRevolveDirection=OFF, sketch=
    mdb.models['TEM'].sketches['__profile__'])
del mdb.models['TEM'].sketches['__profile__']

#----------------------------------------------Define material------------------------------------------------------------

#Isotropic material
mdb.models['TEM'].Material(name='isotropic')
mdb.models['TEM'].materials['isotropic'].Density(table=((1060.0, ), ))
mdb.models['TEM'].materials['isotropic'].Elastic(table=((39765628.0, 0.49), ))

#Anisotropic material
mdb.models['TEM'].Material(name='anisotropic')
mdb.models['TEM'].materials['anisotropic'].Elastic(table=((2460000000.0, 
    2443100000.0, 2460000000.0, 2440000000.0, 2440000000.0, 2530000000.0, 0.0, 
    0.0, 0.0, 8970000.0, 0.0, 0.0, 0.0, 0.0, 8970000.0, 0.0, 0.0, 0.0, 0.0, 
    0.0, 8450000.0), ), type=ANISOTROPIC)
mdb.models['TEM'].materials['anisotropic'].Density(table=((rho, ), ))

#----------------------------------------------Create section---------------------------------------------------------------

#Choose here what material you want to work with
mdb.models['TEM'].HomogeneousSolidSection(material='anisotropic', name=
    'Section-1', thickness=None)
mdb.models['TEM'].parts['ellipsoid'].SectionAssignment(offset=0.0, offsetField=''
    , offsetType=MIDDLE_SURFACE, region=Region(
    cells=mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((a, 0, 
    0), ), )), sectionName='Section-1', thicknessAssignment=
    FROM_SECTION)

#----------------Create the partitions that are to have different material orientations-------------------

#Partition faces
mdb.models['TEM'].parts['ellipsoid'].PartitionCellByPlaneThreePoints(cells=
    mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((a, 0.0, 
    0.0), )), point1=mdb.models['TEM'].parts['ellipsoid'].vertices.findAt((
    sqrt(1-(c/(b-th))**2)*(a-th), c, 0.0), ), point2=
    mdb.models['TEM'].parts['ellipsoid'].vertices.findAt((sqrt(1-(c/b)**2)*a, c, 0.0), )
    , point3=mdb.models['TEM'].parts['ellipsoid'].vertices.findAt((0.0, -b, 
    0.0), ))
mdb.models['TEM'].parts['ellipsoid'].PartitionCellByPlaneThreePoints(cells=
    mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((0.0, 0.0, 
    a), ), ((0.0, 0.0, -a), )), point1=
    mdb.models['TEM'].parts['ellipsoid'].InterestingPoint(
    mdb.models['TEM'].parts['ellipsoid'].edges.findAt((0.0, c, sqrt(1-(c/(b-th))**2)*(a-th)), 
    ), MIDDLE), point2=mdb.models['TEM'].parts['ellipsoid'].InterestingPoint(
    mdb.models['TEM'].parts['ellipsoid'].edges.findAt((0.0, c, -sqrt(1-(c/(b-th))**2)*(a-th)
    ), ), MIDDLE), point3=
    mdb.models['TEM'].parts['ellipsoid'].vertices.findAt((0.0, -b, 0.0), ))

mdb.models['TEM'].ConstrainedSketch(gridSpacing=0.03, name='__profile__', 
    sheetSize=1.56, transform=
    mdb.models['TEM'].parts['ellipsoid'].MakeSketchTransform(
    sketchPlane=mdb.models['TEM'].parts['ellipsoid'].faces.findAt((a-th/2, 
    0.0, 0.0), ), sketchPlaneSide=SIDE1, 
    sketchUpEdge=mdb.models['TEM'].parts['ellipsoid'].edges.findAt((sqrt(1-(c/b)**2)*a-th/2, 
    c, 0.0), ), sketchOrientation=TOP, origin=(0.0, 0.0, 0.0)))
mdb.models['TEM'].parts['ellipsoid'].projectReferencesOntoSketch(filter=
    COPLANAR_EDGES, sketch=mdb.models['TEM'].sketches['__profile__'])

l1 = th/numberOfPartitions

for i in range(numberOfPartitions-1):
    mdb.models['TEM'].sketches['__profile__'].EllipseByCenterPerimeter(axisPoint1=(
        0.0, -b+(i+1)*l1), axisPoint2=(a-(i+1)*l1, 0.0), center=(0.0, 0.0))

mdb.models['TEM'].parts['ellipsoid'].PartitionFaceBySketch(faces=
    mdb.models['TEM'].parts['ellipsoid'].faces.findAt(((a-th/2, 0.0, 
    0.0), ), ), sketch=
    mdb.models['TEM'].sketches['__profile__'], sketchOrientation=TOP, 
    sketchUpEdge=mdb.models['TEM'].parts['ellipsoid'].edges.findAt((sqrt(1-(c/b)**2)*a-th/2,
    c, 0.0), ))
del mdb.models['TEM'].sketches['__profile__']

#Partition cells
for i in range(numberOfPartitions-1):
    mdb.models['TEM'].parts['ellipsoid'].PartitionCellBySweepEdge(cells=
        mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((eps, 0.0, 
        -a+th), )), edges=(mdb.models['TEM'].parts['ellipsoid'].edges.findAt((
        a-(i+1)*l1, 0.0, 0.0), ), ), sweepPath=
        mdb.models['TEM'].parts['ellipsoid'].edges.findAt((eps, c,-sqrt(1-(c/b)**2)*a 
        ), ))
    mdb.models['TEM'].parts['ellipsoid'].PartitionCellBySweepEdge(cells=
        mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((eps, 0.0, 
        a-th), )), edges=(mdb.models['TEM'].parts['ellipsoid'].edges.findAt((
        a-(i+1)*l1, 0.0, 0.0), ), ), sweepPath=
        mdb.models['TEM'].parts['ellipsoid'].edges.findAt((eps, c,sqrt(1-(c/b)**2)*a 
        ), ))
    mdb.models['TEM'].parts['ellipsoid'].PartitionCellBySweepEdge(cells=
        mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((-eps, 0.0, 
        -a+th), )), edges=(mdb.models['TEM'].parts['ellipsoid'].edges.findAt((
        0.0, 0.0, -a+(i+1)*l1), ), ), sweepPath=
        mdb.models['TEM'].parts['ellipsoid'].edges.findAt((-eps, c,-sqrt(1-(c/b)**2)*a 
        ), ))
    mdb.models['TEM'].parts['ellipsoid'].PartitionCellBySweepEdge(cells=
        mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((-eps, 0.0, 
        a-th), )), edges=(mdb.models['TEM'].parts['ellipsoid'].edges.findAt((
        0.0, 0.0, a-(i+1)*l1), ), ), sweepPath=
        mdb.models['TEM'].parts['ellipsoid'].edges.findAt((-eps, c,sqrt(1-(c/b)**2)*a 
        ), ))
    
#--------------------------------------Assign material orientations----------------------------------------------------

for i in range(numberOfPartitions):
    mdb.models['TEM'].parts['ellipsoid'].MaterialOrientation(
        additionalRotationField='', additionalRotationType=ROTATION_ANGLE, angle=
        -30-i*120/(numberOfPartitions-1), axis=AXIS_1, flipNormalDirection=False, flipPrimaryDirection=False, 
        normalAxisDefinition=SURFACE, normalAxisDirection=AXIS_1, normalAxisRegion=
        Region(side1Faces=mdb.models['TEM'].parts['ellipsoid'].faces.findAt(((
        a, 0.0, -eps), ), )), orientationType=DISCRETE, 
        primaryAxisDefinition=EDGE, primaryAxisDirection=AXIS_2, primaryAxisRegion=
        Region(edges=mdb.models['TEM'].parts['ellipsoid'].edges.findAt(((sqrt(1-(c/b)**2)*a, 
        c, -eps), ), )), region=Region(
        cells=mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((a-i*l1-eps, 
        0.0, -eps), ), )), stackDirection=STACK_3)
    mdb.models['TEM'].parts['ellipsoid'].MaterialOrientation(
        additionalRotationField='', additionalRotationType=ROTATION_ANGLE, angle=
        -30-i*120/(numberOfPartitions-1), axis=AXIS_1, flipNormalDirection=False, flipPrimaryDirection=False, 
        normalAxisDefinition=SURFACE, normalAxisDirection=AXIS_1, normalAxisRegion=
        Region(side1Faces=mdb.models['TEM'].parts['ellipsoid'].faces.findAt(((
        a, 0.0, eps), ), )), orientationType=DISCRETE, 
        primaryAxisDefinition=EDGE, primaryAxisDirection=AXIS_2, primaryAxisRegion=
        Region(edges=mdb.models['TEM'].parts['ellipsoid'].edges.findAt(((sqrt(1-(c/b)**2)*a, 
        c, eps), ), )), region=Region(
        cells=mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((a-i*l1-eps, 
        0.0, eps), ), )), stackDirection=STACK_3)
    mdb.models['TEM'].parts['ellipsoid'].MaterialOrientation(
        additionalRotationField='', additionalRotationType=ROTATION_ANGLE, angle=
        -30-i*120/(numberOfPartitions-1), axis=AXIS_1, flipNormalDirection=False, flipPrimaryDirection=False, 
        normalAxisDefinition=SURFACE, normalAxisDirection=AXIS_1, normalAxisRegion=
        Region(side1Faces=mdb.models['TEM'].parts['ellipsoid'].faces.findAt(((
        -a, 0.0, -eps), ), )), orientationType=DISCRETE, 
        primaryAxisDefinition=EDGE, primaryAxisDirection=AXIS_2, primaryAxisRegion=
        Region(edges=mdb.models['TEM'].parts['ellipsoid'].edges.findAt(((-sqrt(1-(c/b)**2)*a, 
        c, -eps), ), )), region=Region(
        cells=mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((-a+i*l1+eps, 
        0.0, -eps), ), )), stackDirection=STACK_3)
    mdb.models['TEM'].parts['ellipsoid'].MaterialOrientation(
        additionalRotationField='', additionalRotationType=ROTATION_ANGLE, angle=
        -30-i*120/(numberOfPartitions-1), axis=AXIS_1, flipNormalDirection=False, flipPrimaryDirection=False, 
        normalAxisDefinition=SURFACE, normalAxisDirection=AXIS_1, normalAxisRegion=
        Region(side1Faces=mdb.models['TEM'].parts['ellipsoid'].faces.findAt(((
        -a, 0.0, eps), ), )), orientationType=DISCRETE, 
        primaryAxisDefinition=EDGE, primaryAxisDirection=AXIS_2, primaryAxisRegion=
        Region(edges=mdb.models['TEM'].parts['ellipsoid'].edges.findAt(((-sqrt(1-(c/b)**2)*a, 
        c, eps), ), )), region=Region(
        cells=mdb.models['TEM'].parts['ellipsoid'].cells.findAt(((-a+i*l1+eps, 
        0.0, eps), ), )), stackDirection=STACK_3)
    
    
#----------------------------------------------Create assembly--------------------------------------------------------------
    
mdb.models['TEM'].rootAssembly.DatumCsysByDefault(CARTESIAN)
mdb.models['TEM'].rootAssembly.Instance(dependent=OFF, name='ellipsoid-1', 
    part=mdb.models['TEM'].parts['ellipsoid'])
    
#------------------------------------------------Create step-------------------------------------------------------------------
    
mdb.models['TEM'].ExplicitDynamicsStep(name='dynamic', previous=
    'Initial', timePeriod=t,  quadBulkViscosity=1.2)

#-----------------------------------------Define load and BCs--------------------------------------------------------------

#BC
mdb.models['TEM'].DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='fix', region=
    Region(
    vertices=mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].vertices.findAt(
    ((0.0, -b, 0.0), ), )), u1=SET, u2=SET, u3=SET, ur1=SET, ur2=SET, ur3=
    SET)

#Create partitions necessary to be able to apply load and at the same time maintain a structured mesh
myDatum1 = mdb.models['TEM'].rootAssembly.DatumPlaneByPrincipalPlane(offset=0.0, 
    principalPlane=XZPLANE)
myDatum3 = mdb.models['TEM'].rootAssembly.DatumPlaneByPrincipalPlane(offset=-c, 
    principalPlane=XZPLANE)
mdb.models['TEM'].rootAssembly.PartitionCellByDatumPlane(cells=
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].cells.getByBoundingBox(
    -2*b,-2*b,-2*b,2*b,2*b,2*b), datumPlane=mdb.models['TEM'].rootAssembly.datums[myDatum1.id])
mdb.models['TEM'].rootAssembly.PartitionCellByDatumPlane(cells=
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].cells.getByBoundingBox(
    -2*b,-2*b,-2*b,2*b,2*b,2*b), datumPlane=mdb.models['TEM'].rootAssembly.datums[myDatum3.id])

#Create surface partition where the load is to be applied
mdb.models['TEM'].ConstrainedSketch(gridSpacing=0.02, name='__profile__', 
    sheetSize=1.0, transform=mdb.models['TEM'].rootAssembly.MakeSketchTransform(
    sketchPlane=mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].faces.findAt(
    (sqrt(1-(c/b)**2)*a-th/2, c, -eps), ), sketchPlaneSide=SIDE1, 
    sketchUpEdge=mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].edges.findAt(
    (sqrt(1-(c/b)**2)*a, c, -eps), ), sketchOrientation=RIGHT, origin=(0.0, c, 
    0.0)))
mdb.models['TEM'].rootAssembly.projectReferencesOntoSketch(filter=COPLANAR_EDGES
    , sketch=mdb.models['TEM'].sketches['__profile__'])
mdb.models['TEM'].sketches['__profile__'].Line(point1=(0.0, 0.0), point2=(
    -sqrt(1-(c/b)**2)*a*cos(pi/16), sqrt(1-(c/b)**2)*a*sin(pi/16)))
mdb.models['TEM'].sketches['__profile__'].Line(point1=(0.0, 0.0), point2=(
    -sqrt(1-(c/b)**2)*a*cos(pi/16), -sqrt(1-(c/b)**2)*a*sin(pi/16)))
mdb.models['TEM'].rootAssembly.PartitionFaceBySketch(faces=
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].faces.getByBoundingBox(
    -a,c-eps,-a,a,c+eps,a ), sketch=
    mdb.models['TEM'].sketches['__profile__'], sketchUpEdge=
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].edges.findAt((
    sqrt(1-(c/b)**2)*a, c, -eps), ))
del mdb.models['TEM'].sketches['__profile__']

 #Determine the load shape in time
mdb.models['TEM'].SmoothStepAmplitude(data=((0.0, 0.0), (1E-4-1E-5, 0.0), (
    1E-4, 1.0), (1E-4+loadTime, 1.0), (1E-4+loadTime+1E-5, 0.0)), name='impulse_smooth', timeSpan=STEP)
    
#Apply surface traction
mdb.models['TEM'].SurfaceTraction(amplitude='impulse_smooth', createStepName=
    'dynamic', directionVector=(
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].vertices.findAt((
    0.0, 0.0, -a), ), 
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].vertices.findAt((
    0.0, 0.0, a), )), distributionType=UNIFORM, field='', 
    localCsys=None, magnitude=F, name='shear', region=Region(
    side1Faces=mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].faces.getByBoundingBox(
    -a,c-eps,-a,-eps,c+eps,a )))

#----------------------------------------------Generate mesh-------------------------------------------------------------

mdb.models['TEM'].rootAssembly.seedPartInstance(deviationFactor=0.1, 
    minSizeFactor=0.1, regions=(
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'], ), size=elSize)
mdb.models['TEM'].rootAssembly.generateMesh(regions=(
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'], ))

#--------------------------------------------------Request outputs---------------------------------------------------------

#Field output
mdb.models['TEM'].fieldOutputRequests['F-Output-1'].setValues(directions=OFF, 
    numIntervals=400, variables=('S', 'SVAVG', 'ENER', ))

#Define sets
mdb.models['TEM'].rootAssembly.Set(name='node1c', vertices=
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].vertices.findAt(((
    -a+l1, 0.0, 0.0), )))
mdb.models['TEM'].rootAssembly.Set(name='node1b', vertices=
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].vertices.findAt(((
    -a+th/2, 0.0, 0.0), )))
mdb.models['TEM'].rootAssembly.Set(name='node1a', vertices=
    mdb.models['TEM'].rootAssembly.instances['ellipsoid-1'].vertices.findAt(((
    -a+(numberOfPartitions-1)*l1, 0.0, 0.0), )))

#History output
mdb.models['TEM'].HistoryOutputRequest(createStepName='dynamic', frequency=10, 
    name='H-Output-1', rebar=EXCLUDE, region=
    mdb.models['TEM'].rootAssembly.sets['node1b'], sectionPoints=
    DEFAULT, variables=('U',))

#----------------------------------------------Create job ---------------------------------------------------------------------

mdb.models['TEM'].rootAssembly.regenerate()
mdb.Job(activateLoadBalancing=False, atTime=None, contactPrint=OFF, 
    description='', echoPrint=OFF, explicitPrecision=SINGLE, historyPrint=OFF, 
    model='TEM', modelPrint=OFF, multiprocessingMode=DEFAULT, name=
    'Truncated_Ellipsoid_Model', nodalOutputPrecision=SINGLE, numCpus=1, numDomains=1, 
    parallelizationMethodExplicit=DOMAIN, queue=None, scratch='', type=ANALYSIS
    , userSubroutine='', waitHours=0, waitMinutes=0)
