#This is a python script that creates the FEM model that illustrates the wave surface in the
#theory capter in the master thesis. Under the section "Define variables", the user can specify 
#for example the geometry or mesh density. Note that this model has a different material
#than the other models

#Written by Erik Grimsmo

#------------------------------------------------------Initialize---------------------------------------------------------------

from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *

session.journalOptions.setValues(replayGeometry=COORDINATE,recoverGeometry=COORDINATE)

#Change work directory (use your personal directory, and uncomment the line below)
#os.chdir(r'D:\abaqusworkdirectory')

#----------------------------------------------Define variables--------------------------------------------------------------

inner = 75 #inner radius
outer = 3300 #outer radius

rho = 1060.0 #Density

F = 100 #Amplitude of applied pressure

loadTime = 2.0 #Duration of load
t = 1.1 #Period of time integration
n = 30 #Number of elements in seed

#----------------------------------------------Create model---------------------------------------------------------------

eps = 0.00001 #A variable that helps with modeling

mdb.Model(name='WSM')
mdb.models['WSM'].setValues(noPartsInputFile=ON)

#----------------------------------------------Create sketch---------------------------------------------------------------

mdb.models['WSM'].ConstrainedSketch(name='__profile__', sheetSize=
    1.0)
mdb.models['WSM'].sketches['__profile__'].CircleByCenterPerimeter(
    center=(0.0, 0.0), point1=(inner, 0.0))
mdb.models['WSM'].sketches['__profile__'].CircleByCenterPerimeter(
    center=(0.0, 0.0), point1=(outer, 0.0))
mdb.models['WSM'].Part(dimensionality=TWO_D_PLANAR, name='circularPlate', 
    type=DEFORMABLE_BODY)
mdb.models['WSM'].parts['circularPlate'].BaseShell(sketch=
    mdb.models['WSM'].sketches['__profile__'])
del mdb.models['WSM'].sketches['__profile__']

#----------------------------------------------Define material------------------------------------------------------------

mdb.models['WSM'].Material(name='anisotropic')
mdb.models['WSM'].materials['anisotropic'].Elastic(table=((2460000000.0, 
        2440000000.0, 10000000000.0, 2443100000.0, 2440000000.0, 2460000000.0, 0.0, 
        0.0, 0.0, 8970000.0, 0.0, 0.0, 0.0, 0.0, 8450000.0, 0.0, 0.0, 0.0, 0.0, 
        0.0, 8970000), ), type=ANISOTROPIC)
mdb.models['WSM'].materials['anisotropic'].Density(table=((rho, ), ))
mdb.models['WSM'].parts['circularPlate'].MaterialOrientation(
    additionalRotationField='', additionalRotationType=ROTATION_ANGLE, angle=
    0, axis=AXIS_3, fieldName='', localCsys=None, orientationType=SYSTEM, 
    region=Region(
    faces=mdb.models['WSM'].parts['circularPlate'].faces.findAt(((outer-eps, 
    0.0, 0.0), ), )), stackDirection=STACK_3)

#----------------------------------------------Create section---------------------------------------------------------------

mdb.models['WSM'].HomogeneousSolidSection(material='anisotropic', name=
    'Section-1', thickness=None)
mdb.models['WSM'].parts['circularPlate'].SectionAssignment(offset=0.0, 
    offsetField='', offsetType=MIDDLE_SURFACE, region=Region(
    faces=mdb.models['WSM'].parts['circularPlate'].faces.getSequenceFromMask(
    mask=('[#1 ]', ), )), sectionName='Section-1', thicknessAssignment=
    FROM_SECTION)

#----------------------------------------------Partition face--------------------------------------------------------------

mdb.models['WSM'].ConstrainedSketch(gridSpacing=0.001, name=
    '__profile__', sheetSize=0.056, transform=
    mdb.models['WSM'].parts['circularPlate'].MakeSketchTransform(
    sketchPlane=mdb.models['WSM'].parts['circularPlate'].faces.findAt(
    (inner, inner, 0.0), (0, 0, 1.0)), sketchPlaneSide=SIDE1, 
    sketchOrientation=RIGHT, origin=(0.0, 0.0, 0.0)))
mdb.models['WSM'].parts['circularPlate'].projectReferencesOntoSketch(
    filter=COPLANAR_EDGES, sketch=
    mdb.models['WSM'].sketches['__profile__'])
mdb.models['WSM'].sketches['__profile__'].Line(point1=(inner, 0.0), 
    point2=(outer, 0.0))
mdb.models['WSM'].sketches['__profile__'].Line(point1=(-inner, 0.0), 
    point2=(-outer, 0.0))
mdb.models['WSM'].sketches['__profile__'].Line(point1=(0.0, inner), 
    point2=(0, outer))
mdb.models['WSM'].sketches['__profile__'].Line(point1=(0.0, -inner), 
    point2=(0, -outer))
    
l1 = inner + (outer-inner)/10 #Length used in next partitions

mdb.models['WSM'].sketches['__profile__'].ArcByCenterEnds(center=(
    0.0, 0.0), direction=COUNTERCLOCKWISE, point1=(l1, 0), point2=(0, 
    l1))
mdb.models['WSM'].sketches['__profile__'].ArcByCenterEnds(center=(
    0.0, 0.0), direction=COUNTERCLOCKWISE, point1=(0, l1), point2=(-l1, 
    0))
mdb.models['WSM'].sketches['__profile__'].ArcByCenterEnds(center=(
    0.0, 0.0), direction=COUNTERCLOCKWISE, point1=(-l1, 0), point2=(0, 
    -l1))    
mdb.models['WSM'].sketches['__profile__'].ArcByCenterEnds(center=(
    0.0, 0.0), direction=COUNTERCLOCKWISE, point1=(0.0, -l1), point2=(l1, 
    0))      
mdb.models['WSM'].parts['circularPlate'].PartitionFaceBySketch(
    faces=mdb.models['WSM'].parts['circularPlate'].faces.findAt(((
    inner, inner, 0.0), )), sketch=
    mdb.models['WSM'].sketches['__profile__'])
del mdb.models['WSM'].sketches['__profile__']

#----------------------------------------------Create assembly--------------------------------------------------------------

mdb.models['WSM'].rootAssembly.DatumCsysByDefault(CARTESIAN)
mdb.models['WSM'].rootAssembly.Instance(dependent=ON, name='circularPlate-1', 
    part=mdb.models['WSM'].parts['circularPlate'])

#------------------------------------------------Create step-------------------------------------------------------------------

mdb.models['WSM'].ExplicitDynamicsStep(name='dynamic', previous=
    'Initial', timePeriod=t,  quadBulkViscosity=0.0)

#------------------------------------------Apply BCs and load---------------------------------------------------------------

#Determine the load shape in time
mdb.models['WSM'].SmoothStepAmplitude(data=((0.0, 0.0), (1E-4-1E-5, 0.0), (
     1E-4, 1.0), (1E-4+loadTime, 1.0), (1E-4+loadTime+1E-5, 0.0)), name='impulse_smooth', timeSpan=STEP)

#Apply pressure    
mdb.models['WSM'].Pressure(amplitude='impulse_smooth', createStepName=
    'dynamic', distributionType=UNIFORM, field='', magnitude=F, name=
    'pressure', region=Region(
    side1Edges=mdb.models['WSM'].rootAssembly.instances['circularPlate-1'].edges.findAt(
    ((inner/sqrt(2), inner/sqrt(2), 0.0), ), ((-inner/sqrt(2), inner/sqrt(2), 0.0), ),
    ((-inner/sqrt(2), -inner/sqrt(2), 0.0), ), ((inner/sqrt(2), -inner/sqrt(2), 0.0), ),)))
    
    
#Fixed outer boundary
mdb.models['WSM'].DisplacementBC(amplitude=UNSET, createStepName=
    'Initial', distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'fixed1', region=Region(
    edges=mdb.models['WSM'].rootAssembly.instances['circularPlate-1'].edges.findAt(
    ((-outer/sqrt(2), outer/sqrt(2), 0.0), ), )), u1=SET, u2=SET, ur3=SET)
mdb.models['WSM'].DisplacementBC(amplitude=UNSET, createStepName=
    'Initial', distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'fixed2', region=Region(
    edges=mdb.models['WSM'].rootAssembly.instances['circularPlate-1'].edges.findAt(
    ((outer/sqrt(2), outer/sqrt(2), 0.0), ), )), u1=SET, u2=SET, ur3=SET)    
mdb.models['WSM'].DisplacementBC(amplitude=UNSET, createStepName=
    'Initial', distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'fixed3', region=Region(
    edges=mdb.models['WSM'].rootAssembly.instances['circularPlate-1'].edges.findAt(
    ((outer/sqrt(2), -outer/sqrt(2), 0.0), ), )), u1=SET, u2=SET, ur3=SET)
mdb.models['WSM'].DisplacementBC(amplitude=UNSET, createStepName=
    'Initial', distributionType=UNIFORM, fieldName='', localCsys=None, name=
    'fixed4', region=Region(
    edges=mdb.models['WSM'].rootAssembly.instances['circularPlate-1'].edges.findAt(
    ((-outer/sqrt(2), -outer/sqrt(2), 0.0), ), )), u1=SET, u2=SET, ur3=SET)


#----------------------------------------------Generate mesh-------------------------------------------------------------

mdb.models['WSM'].parts['circularPlate'].seedEdgeByNumber(
        constraint=FINER, edges=
        mdb.models['WSM'].parts['circularPlate'].edges.findAt(
        ((inner/sqrt(2), inner/sqrt(2), 0.0), ),((-inner/sqrt(2), inner/sqrt(2), 0.0), ),
        ((-inner/sqrt(2), -inner/sqrt(2), 0.0), ),((inner/sqrt(2), -inner/sqrt(2), 0.0), ),
        ((l1/sqrt(2), l1/sqrt(2), 0.0), ),((-l1/sqrt(2), l1/sqrt(2), 0.0), ),
        ((-l1/sqrt(2), -l1/sqrt(2), 0.0), ),((l1/sqrt(2), -l1/sqrt(2), 0.0), ),        
        ((outer/sqrt(2), outer/sqrt(2), 0.0), ),((-outer/sqrt(2), outer/sqrt(2), 0.0), ),
        ((-outer/sqrt(2), -outer/sqrt(2), 0.0), ),((outer/sqrt(2), -outer/sqrt(2), 0.0), ),        ), number=n)    
mdb.models['WSM'].parts['circularPlate'].setMeshControls(elemShape=
    QUAD, regions=
    mdb.models['WSM'].parts['circularPlate'].faces.getByBoundingSphere(
    center=(0,0,0), radius=2*outer), technique=STRUCTURED)
mdb.models['WSM'].parts['circularPlate'].setElementType(elemTypes=(
    ElemType(elemCode=CPE4R, elemLibrary=EXPLICIT, secondOrderAccuracy=OFF, 
    hourglassControl=DEFAULT, distortionControl=DEFAULT), ElemType(
    elemCode=CPE3, elemLibrary=EXPLICIT)), regions=(
    mdb.models['WSM'].parts['circularPlate'].faces.getByBoundingSphere(
    center=(0,0,0), radius=2*outer), ))
mdb.models['WSM'].parts['circularPlate'].generateMesh()

#--------------------------------------------------Request outputs---------------------------------------------------------

#Create interesting sets
mdb.models['WSM'].parts['circularPlate'].Set(name='horizontalPoint'
    , vertices=
    mdb.models['WSM'].parts['circularPlate'].vertices.findAt(((
    l1, 0.0, 0.0), )))
mdb.models['WSM'].parts['circularPlate'].Set(name='verticalPoint', 
    vertices=
    mdb.models['WSM'].parts['circularPlate'].vertices.findAt(((0.0, 
    l1, 0.0), )))

#Field output
mdb.models['WSM'].fieldOutputRequests['F-Output-1'].setValues(numIntervals=
    400, variables=('S', 'SVAVG', 'SENER' ))

#History output
mdb.models['WSM'].historyOutputRequests['H-Output-1'].setValues(
    frequency=10, rebar=EXCLUDE, region=
    mdb.models['WSM'].rootAssembly.instances['circularPlate-1'].sets['horizontalPoint']
    , sectionPoints=DEFAULT, variables=('S11', 'S12', 'MISES', 'UT'))
mdb.models['WSM'].HistoryOutputRequest(createStepName='dynamic', 
    frequency=10, name='H-Output-2', rebar=EXCLUDE, region=
    mdb.models['WSM'].rootAssembly.instances['circularPlate-1'].sets['verticalPoint']
    , sectionPoints=DEFAULT, variables=('S11', 'S12', 'MISES', 'UT'))

#----------------------------------------------Create job ---------------------------------------------------------------------

mdb.models['WSM'].rootAssembly.regenerate()
mdb.Job(activateLoadBalancing=False, atTime=None, contactPrint=OFF, 
    description='', echoPrint=OFF, explicitPrecision=SINGLE, historyPrint=OFF, 
    model='WSM', modelPrint=OFF, multiprocessingMode=DEFAULT, name=
    'Wave_Surface_Model', nodalOutputPrecision=SINGLE, numCpus=1, numDomains=
    1, parallelizationMethodExplicit=DOMAIN, queue=None, scratch='', type=
    ANALYSIS, userSubroutine='', waitHours=0, waitMinutes=0)

